/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            GL_Scene.h                                      //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLSCENE_H
#define GLSCENE_H

#include "GL_X3DGroupingNode.h"

namespace X3DTK {
namespace GL {

/// Base class of the Core component. A set of X3DGLNode represents a scene graph
/// related with the X3D scene graph. Each X3DNode able to be drawn directly has 
/// its X3DGLNode correspondence. For example X3DIndexedFaceSet is related with 
/// GLIndexedFaceSet, whereas Coordinate has no X3DGLNode correspondent. 

class Scene : public X3DGroupingNode
{
public:
  /// Constructor.
  Scene();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~Scene();

  /// Sets the filename of the Scene.
  void setFileName(const SFString &file);
 
  /// Gets the name of the file.
  inline const SFString &getFileName() const {return _fileName;};

  /// Updates the attributes of the node.
  virtual void update();

protected:
  /// Copy constructor.
  Scene(const Scene &S);

private:  
  /// File attribute.
  SFString _fileName;
};

}
}

#endif
