/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            MeshTypes.h                                     //
////////////////////////////////////////////////////////////////////////////////

#ifndef MESH_SceneGraphTYPES_H
#define MESH_SceneGraphTYPES_H

#include "Platform.h"

#include <vector>
#include <list>
#include <iostream>

namespace X3DTK {
namespace MESH {

class X3DNode;
typedef X3DNode *SFNode;
typedef std::list<SFNode> MFNode;

template<class A, class B, class C> class SFTemplateVertex;
template<class A, class B, class C> class SFTemplateEdge;
template<class A, class B, class C> class SFTemplateFace;

/// Class providing a vector of Vertex for convenience, as well as object iterators.

template<class VData, class EData, class FData>
class MFTemplateVertex : public std::vector<SFTemplateVertex<VData, EData, FData> *>
{
public:
  /// Constructor.
  MFTemplateVertex() : std::vector<SFTemplateVertex<VData, EData, FData> *>() {};
  /// Constructor from a vector.
  explicit MFTemplateVertex(const std::vector<SFTemplateVertex<VData, EData, FData> *> &vector)
  : std::vector<SFTemplateVertex<VData, EData, FData> *>(vector)
  {}; 
};

/// Class providing a vector of Edge for convenience, as well as object iterators.

template<class EData, class FData, class VData>
class MFTemplateEdge : public std::vector<SFTemplateEdge<EData, FData, VData> *>
{
public:
  /// Constructor.
  MFTemplateEdge() : std::vector<SFTemplateEdge<EData, FData, VData> *>() {};
  /// Constructor from a vector.
  explicit MFTemplateEdge(const std::vector<SFTemplateEdge<EData, FData, VData> *> &vector, bool ccw)
  : std::vector<SFTemplateEdge<EData, FData, VData> *>(vector)
  {
    if (!ccw)
    {
      unsigned int last = vector.size() - 1;
      unsigned int mid = (last + 1)/2;
      for (unsigned int i = 0; i < mid; ++i)
      {
        SFTemplateEdge<EData, FData, VData> *temp = (*this)[i];
        (*this)[i] = (*this)[last - i];
        (*this)[last - i] = temp;
      }
    }  
  };
};

/// Class providing a vector of Face for convenience, as well as object iterators.

template<class FData, class VData, class EData>
class MFTemplateFace : public std::vector<SFTemplateFace<FData, VData, EData> *>
{
public:
  /// Constructor.
  MFTemplateFace() : std::vector<SFTemplateFace<FData, VData, EData> *>() {};
  /// Constructor from a vector.
  explicit MFTemplateFace(const std::vector<SFTemplateFace<FData, VData, EData> *> &vector)
  : std::vector<SFTemplateFace<FData, VData, EData> *>(vector)
  {};
};

}
}

#endif
