/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            X3DComponentWalker.h                            //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DCOMPONENTWALKER_H
#define X3DCOMPONENTWALKER_H

#include "X3DComponent.h"
#include "WalkingFunction.h"
#include "X3DAbstractNode.h"
#include "Walker.h"
#include "Visitor.h"
#include "SFSceneGraph.h"

namespace X3DTK {

/// Abstract class defining the concept of component walker.

class X3DComponentWalker : public X3DComponent
{
public:
  /// Constructor.
  X3DComponentWalker();
  /// Destructor.
  virtual ~X3DComponentWalker() = 0;
  
  /// Sets the NodeVisitingProxy
  void setWalker(Walker *walker);
  /// Sets the NodeVisitingProxy
  void setVisitor(Visitor *visitor);
  
  /// Gets the walking function of name.
  WalkingFunction *getWalkingFunctionOf(const SFType *type) const;
  
protected:  
  /// Reference to the NodeWalkingProxy.
  Walker *walker;
  /// Reference to the NodeVisitingProxy.
  Visitor *visitor;
  /// Defines a new creation function.
  template<class C, class T> 
  void defineWalkingFunction(void (C::*ptrF)(T *) const)
  {
    SFType::beginFunctionDefinition();
    WalkingFunction *WF = new WalkingFunction(reinterpret_cast<ptrToWalkingFunction>(ptrF), this);
    //finding if an occurence of T
    T *t = new T();
    SFString name = t->getTypeName();
    SFComponent *TComponent = t->getType()->getComponent();
    //checking if the node belongs to the component.
    if (TComponent != component)
    {
      SFString sg1, sg2;
      if (component->getSceneGraphName() != "")
        sg1 = " of the " + component->getSceneGraphName() + " scene graph";
      if (t->getType()->getSceneGraphName() != "")
        sg1 = " of the " + t->getType()->getSceneGraphName() + " scene graph";  

      cx3d << "warning: defineWalkingFunction for " << name << " which doesn't belong to the " << component->getName() << " component" << sg1 << "," << std::endl;
      cx3d << "         but to the " << t->getType()->getComponentName() << " component" << sg2 << std::endl;    
    }         
    else
      _walkingDict.insert(std::pair<SFString, WalkingFunction *>(t->getType()->getEncodedName(), WF));
    
    delete t;       
    SFType::endFunctionDefinition();
  };
  
private:
  WalkingDict _walkingDict;
};

}

#endif
