/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            X3D_TemplateMeshBuilderRenderingVisitor.h       //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEMESHBUILDERRENDERINGVISITOR_H
#define TEMPLATEMESHBUILDERRENDERINGVISITOR_H

#include "X3D_RenderingVisitor.h"
#include "X3D_TemplateMeshBuilderStateVariables.h"
#include "X3D_TemplateMeshBuilderTraits.h"
#include "X3D_Coordinate.h"

namespace X3DTK {
namespace X3D {

/// Visitor for the Rendering component of the TemplateMeshBuilder module.

template<class VData, class EData, class FData>
class TemplateMeshBuilderRenderingVisitor : public RenderingVisitor
{
public:
  /// Constructor.
  TemplateMeshBuilderRenderingVisitor()
  : RenderingVisitor()
  {
    defineEnterFunction(&TemplateMeshBuilderRenderingVisitor::enterCoordinate);
  
    // StateVariables assignation.
    stateVariables = GraphTraversal::getInstanceOf<TemplateMeshBuilderStateVariables>();
  };
  
  /// Destructor.
  virtual ~TemplateMeshBuilderRenderingVisitor()
  {
  };

  /// Enters a Coordinate.
  virtual void enterCoordinate(Coordinate *C) const
  {
    MESH::TemplateVertexSet<VData, EData, FData> *MC = dynamic_cast<MESH::TemplateVertexSet<VData, EData, FData> *>(stateVariables->getNode(C));
    if (MC == 0)
    {
       MC = new MESH::TemplateVertexSet<VData, EData, FData>();
       stateVariables->addCoupleNode(C, MC);

       // Filling the Vertices.
      const MFVec3f &point = C->getPoint();
    
      for (MFVec3f::const_iterator it = point.begin(); it != point.end(); ++it)
        MC->createVertex();
      
      TemplateMeshBuilderCoordinateTraits<VData, EData, FData, Coordinate>::process(MC, C);    
    } 
    stateVariables->pushNode(MC);
  };
  
protected:
  TemplateMeshBuilderStateVariables *stateVariables;
};

}
}

#endif
