/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            MESH_SFTemplateEdgeContent.h                    //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEEDGECONTENT_H
#define TEMPLATEEDGECONTENT_H

#include "MESH_SceneGraphTypes.h"

#include <vector>

namespace X3DTK {
namespace MESH {

template<class A, class B, class C> class SFTemplateVertex;
template<class A, class B, class C> class SFTemplateFace;

/// Template class providing the Edge content for the Mesh structure. Indeed these are 
/// datas that the two symetric oriented-edges share.

template<class EData, class FData, class VData>
class SFTemplateEdgeContent
{
  template<class A, class B, class C, class D> friend class TemplateMesh;
  template<class A, class B, class C> friend class SFTemplateEdge;
public:
  /// Gets the Vertex from.
  inline SFTemplateVertex<VData, EData, FData> *getFromVertex() const {return _from;};
  /// Gets the Vertex to.
  inline SFTemplateVertex<VData, EData, FData> *getToVertex() const {return _to;};
  /// Gets the 1-faces.
  inline const MFTemplateFace<FData, VData, EData> &get1Faces() const {return _r1Faces;};
  /// Gets the 2-faces.
  inline const MFTemplateFace<FData, VData, EData> &get2Faces() const {return _r2Faces;};
  /// Returns true if is a boundary.
  inline bool isBoundary() const 
  {
    return _isBoundary;
  };
  /// Gets the data.
  inline EData &data() {return _data;};
  /// Gets the data.
  inline const EData &data() const {return _data;};

private:
  SFTemplateVertex<VData, EData, FData> *const _from;
  SFTemplateVertex<VData, EData, FData> *const _to;
  MFTemplateFace<FData, VData, EData> _r1Faces;
  MFTemplateFace<FData, VData, EData> _r2Faces;
  bool _isBoundary;
  EData _data;
  
  /// Constructor.
  SFTemplateEdgeContent(SFTemplateVertex<VData, EData, FData> * from, SFTemplateVertex<VData, EData, FData> *const to) 
  : _from(from), _to(to), _isBoundary(true)
  {};
  
  /// Destructor.
  ~SFTemplateEdgeContent() {};
    
  /// Adds a 1-face.
  void add1Face(SFTemplateFace<FData, VData, EData> *face) 
  {
    _r1Faces.push_back(face);
    if (_r2Faces.size() > 0)
      _isBoundary = false;
  };
  /// Adds a 2-face.
  void add2Face(SFTemplateFace<FData, VData, EData> *face) 
  {
    _r2Faces.push_back(face);
    if (_r1Faces.size() > 0)
      _isBoundary = false;
  };

};

}
}

#endif
