/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            X3DMeshGroupingNode.h                           //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef X3DMESHGROUPINGNODE
#define X3DMESHGROUPINGNODE

#include "X3D_SceneGraphTypes.h"
#include "MESH_X3DNode.h"

namespace X3DTK {
namespace MESH {

/// Base class defining a Grouping Node for the Mesh graph scene.

class X3DGroupingNode : public X3DNode
{
public:
  /// Constructor.
  X3DGroupingNode();
  /// Virtual destructor.
  virtual ~X3DGroupingNode();

  /// Adds an X3DNode.
  void addX3DNodeChild(X3D::X3DNode *N);
  
  /// Gets the list of X3DNodes.
  inline X3D::MFNode getX3DNodeChildren() const {return _x3dChildren;};
  /// Gets the children list.
  virtual MFAbstractNode getChildrenList() const;
  
  /// Adds a child to the Children attribute. Returns TRUE if succeeds.
  bool addChild(const SFAbstractNode &N);
  /// Sets the child. Behaves like addChild.
  bool setChild(const SFAbstractNode &N);
  /// Removes the child. Return TRUE if succeeds.
  bool removeChild(const SFAbstractNode &N);
  
  /// Gets the Children attribute.
  inline MFNode getChildren() const {return _children;};
  
protected:
  /// Copy constructor.		  
  X3DGroupingNode(const X3DGroupingNode &N);

private:  
  MFNode _children;
  X3D::MFNode _x3dChildren;
};

}
}

#endif
