#include "MESH_MeshEdgesDrawerCoreVisitor.h"
#include "MESH_Transform.h"

#include <GL/gl.h>
#include <iostream>

using namespace std;

namespace X3DTK {
namespace MESH {

MeshEdgesDrawerCoreVisitor::MeshEdgesDrawerCoreVisitor()
{
  // Enter and leave functions.
  defineEnterFunction(&MeshEdgesDrawerCoreVisitor::enterTransform);
  defineEnterFunction(&MeshEdgesDrawerCoreVisitor::enterMesh);
  defineLeaveFunction(&MeshEdgesDrawerCoreVisitor::leaveTransform);
  
  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<MeshEdgesDrawerStateVariables>();
}

void MeshEdgesDrawerCoreVisitor::enterTransform(Transform *T) const
{
  glMatrixMode(GL_MODELVIEW);
  stateVariables->pushMatrix(T->getTransform());
  glPushMatrix();
  glMultMatrixf(T->getTransform().toFloat16());
}

void MeshEdgesDrawerCoreVisitor::leaveTransform(Transform *T) const
{
  glMatrixMode(GL_MODELVIEW);
  stateVariables->popMatrix();
  glPopMatrix();
}

void MeshEdgesDrawerCoreVisitor::enterMesh(Mesh *M) const
{
  stateVariables->storeMesh(M);
  int meshId = stateVariables->getCurrentMeshId();

  if (stateVariables->getSelectionMode())
    glPushName(meshId);

  glColor3f(1.0f, 0.9f, 0.8f);
  glLineWidth(1.0f);
  glDisable(GL_LIGHTING);  
  
  const Mesh::MFEdge &edges = M->getEdges();

  int index = 0;
  for (Mesh::MFEdge::const_iterator e = edges.begin(), end = edges.end() ; e != end; ++e)
  {
    if (stateVariables->getSelectionMode())
      glPushName(index++);

    if ((*e)->isBoundary())
      glColor3f(0.3f, 0.3f, 0.9f);
    else
      glColor3f(0.7f, 0.7f, 0.7f);

    glBegin(GL_LINES);
    SFPoint3f point = (*e)->getFromVertex()->data().getPoint();
    glVertex3fv(point.f_data());
    point = (*e)->getToVertex()->data().getPoint();
    glVertex3fv(point.f_data());
    glEnd();
        
    if (stateVariables->getSelectionMode())
      glPopName();
  }
  if (stateVariables->getSelectionMode())
      glPopName();
}

}
}
