#include "MESH_MeshFacesDrawerCoreVisitor.h"
#include "MESH_Transform.h"

#include <GL/gl.h>
#include <iostream>

using namespace std;

namespace X3DTK {
namespace MESH {

MeshFacesDrawerCoreVisitor::MeshFacesDrawerCoreVisitor()
{
  // Enter and leave functions.
  defineEnterFunction(&MeshFacesDrawerCoreVisitor::enterTransform);
  defineEnterFunction(&MeshFacesDrawerCoreVisitor::enterMesh);
  defineLeaveFunction(&MeshFacesDrawerCoreVisitor::leaveTransform);
  
  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<MeshFacesDrawerStateVariables>();
}

void MeshFacesDrawerCoreVisitor::enterTransform(Transform *T) const
{
  glMatrixMode(GL_MODELVIEW);
  stateVariables->pushMatrix(T->getTransform());
  glPushMatrix();
  glMultMatrixf(T->getTransform().toFloat16());
}

void MeshFacesDrawerCoreVisitor::leaveTransform(Transform *T) const
{
  glMatrixMode(GL_MODELVIEW);
  stateVariables->popMatrix();
  glPopMatrix();
}

void MeshFacesDrawerCoreVisitor::enterMesh(Mesh *M) const
{
  glEnable(GL_LIGHTING);
  
  stateVariables->storeMesh(M);
  int meshId = stateVariables->getCurrentMeshId();

  if (stateVariables->getSelectionMode())
    glPushName(meshId);

  bool colorPerVertex = M->data().getColorPerVertex();
  bool normalPerVertex = M->data().getNormalPerVertex();
  
  const Mesh::MFFace &faces = M->getFaces();

  if (stateVariables->getOneColorPerMesh())
    glColor3f(0.05f*((3+meshId*3)%10) + 0.5f, 0.05f*((5+meshId*4)%10) + 0.5f, 0.05f*((7+meshId*5)%10) + 0.5f);

  for (Mesh::MFFace::const_iterator f = faces.begin(), end = faces.end() ; f != end; ++f)
  {
    if (!normalPerVertex)
    {
      SFVec3f normal = (*f)->data().getNormal();
      glNormal3fv(normal.f_data());
    }

    if ((!colorPerVertex) && (!stateVariables->getOneColorPerMesh()))
    {
      SFColorRGBA color = (*f)->data().getColor();
      glColor4f(color.r, color.g, color.b, color.a);
    }
        
    if (stateVariables->getSelectionMode())
      glPushName((*f)->getIndex());

    glBegin(GL_POLYGON);
    const Mesh::MFEdge &edges = (*f)->getEdges();
    for (Mesh::MFEdge::const_iterator e = edges.begin(), end = edges.end() ; e != end; ++e)
    {
      if (normalPerVertex)
      {
        SFVec3f normal = (*e)->getFromVertex()->data().getNormalOfFace(*f);
        glNormal3fv(normal.f_data());
      }
      if ((colorPerVertex) && (!stateVariables->getOneColorPerMesh()))
      {
        SFColorRGBA color = (*e)->getFromVertex()->data().getColorOfFace(*f);
        glColor4f(color.r, color.g, color.b, color.a);
      }
      
      SFPoint3f point = (*e)->getFromVertex()->data().getPoint();
      glVertex3fv(point.f_data());
    }
    glEnd();

    if (stateVariables->getSelectionMode())
      glPopName();
  }

  if (stateVariables->getSelectionMode())
    glPopName();
    
  glDisable(GL_LIGHTING);  
}

}
}
