#include "X3D_SceneSaver.h"
#include "X3D_DFSGraphTraversal.h"
#include "X3D_SceneSaverCoreVisitor.h"

#include <fstream>
#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

SceneSaver::SceneSaver()
: X3DSceneSaver()
{
  stateVariables = GraphTraversal::getInstanceOf<SceneSaverStateVariables>();

  graphTraversal = new DFSGraphTraversal();  
  graphTraversal->setComponentVisitor(new SceneSaverCoreVisitor());
}

SceneSaver::~SceneSaver()
{
  GraphTraversal::removeInstanceOf<SceneSaverStateVariables>();

  delete graphTraversal;
}

void SceneSaver::saveAs(Scene *S, const SFString &fileName) const
{
  if (S == 0)
  {
    cx3d << "unable to write to file, because the Scene is NULL!" << endl;
    return;
  }
    
  ofstream output(fileName);
  
  output << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
  output << "<X3D>" << endl;
  
  graphTraversal->traverse(S);
  MFString text = stateVariables->getOutput();
  
  for (MFString::const_iterator it = text.begin(); it != text.end(); ++it)
    output << (*it) << endl;

  output << "</X3D>" << endl;
}

void SceneSaver::save(Scene *S) const
{
  saveAs(S, S->getFileName());
}

}
}
