#include "X3D_SceneSaverStateVariables.h"
#include "X3DAbstractNode.h"
#include "X3D_X3DNode.h"

using namespace std;

namespace X3DTK {
namespace X3D {

SceneSaverStateVariables::SceneSaverStateVariables()
: StateVariables(), _lineNumber(-1), _tab("  ")
{
}

SceneSaverStateVariables::~SceneSaverStateVariables()
{
}

void SceneSaverStateVariables::pushHasUSE(bool value)
{
  _hasUSEStack.push_front(value);
}

void SceneSaverStateVariables::newLine()
{
  ++_lineNumber;
  _output.push_back(SFString(_tab));
}

void SceneSaverStateVariables::pushCurrentLine()
{
  _lineStack.push_front(_lineNumber);
}

SFString &SceneSaverStateVariables::getCurrentLine()
{
  return _output.back();
}

SFString &SceneSaverStateVariables::getOpeningLine()
{
  return _output[_lineStack.front()];
}

void SceneSaverStateVariables::addTab()
{
  _tab += "  ";
}

void SceneSaverStateVariables::removeTab()
{
  _tab.erase(0, 2);
}

bool SceneSaverStateVariables::hasChild()
{
  return (_lineNumber == _lineStack.front());
}

void SceneSaverStateVariables::popHasUSE()
{
  _hasUSEStack.pop_front();
}

void SceneSaverStateVariables::popLine()
{
  _lineStack.pop_front();
}

SFString SceneSaverStateVariables::getDEF(const SFNode &N)
{
  //finding name
  SFString DEF = X3DAbstractNode::DEFDict.getNameOfNode(N);
  if (DEF == "")
    return "";
  
  //the DEF doesn't exist
  //inserting the new DEF
  bool found = false;
  for (list<SFString>::const_iterator it = _DEFList.begin(); it != _DEFList.end(); ++it)
    if (*it == DEF)
    {
      found = true;
      break;
    }  
    
  if (!found)
  {
    _DEFList.push_back(DEF);
    return SFString(" DEF=\"") + DEF + "\"";  
  }
  return ""; 
}

SFString SceneSaverStateVariables::getUSE(const SFNode &N) const
{
  //finding name
  SFString DEF = X3DAbstractNode::DEFDict.getNameOfNode(N);
  if (DEF == "")
    return "";
  
  bool found = false;
  for (list<SFString>::const_iterator it = _DEFList.begin(); it != _DEFList.end(); ++it)
    if (*it == DEF)
    {
      found = true;
      break;
    }  
    
  if (found)
    return SFString(" USE=\"") + DEF + "\"";

  return "";  
}

}
}
