#include "X3D_WorldCoordTranslatorRenderingVisitor.h"
#include "X3D_Coordinate.h"
#include "X3D_Normal.h"
#include "X3D_X3DGeometryNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {


WorldCoordTranslatorRenderingVisitor::WorldCoordTranslatorRenderingVisitor()
: RenderingVisitor()
{
  // Enter functions.
  defineEnterFunction(&WorldCoordTranslatorRenderingVisitor::enterCoordinate);
  defineEnterFunction(&WorldCoordTranslatorRenderingVisitor::enterNormal);

  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<WorldCoordTranslatorStateVariables>();
}

WorldCoordTranslatorRenderingVisitor::~WorldCoordTranslatorRenderingVisitor()
{
}

void WorldCoordTranslatorRenderingVisitor::enterCoordinate(Coordinate *C) const
{
  SFMatrix34f transform = stateVariables->getMatrix();
  
  MFVec3f vertexArray = C->getPoint();
  for (MFVec3f::iterator itVertex = vertexArray.begin(); itVertex != vertexArray.end(); ++itVertex)
    *itVertex = SFVec3f(transform*SFPoint3f(*itVertex));

  C->setPoint(vertexArray);  
}

void WorldCoordTranslatorRenderingVisitor::enterNormal(Normal *N) const
{
  SFMatrix34f transform = stateVariables->getMatrix();
  
  MFVec3f normalArray = N->getVector();
  for (MFVec3f::iterator itNormal = normalArray.begin(); itNormal != normalArray.end(); ++itNormal)
  {
    *itNormal = (transform*(*itNormal));
    (*itNormal).normalize();
  }  

  N->setVector(normalArray);  
}

}
}
