#include "MemReleaser.h"
#include "X3D_DFSGraphTraversal.h"
#include "GL_DFSGraphTraversal.h"
#include "MESH_DFSGraphTraversal.h"
#include "MemReleaserGenericAbstractVisitor.h"
#include "MESH_MemReleaserCoreVisitor.h"

#include <iostream>

using namespace std;

namespace X3DTK {

MemReleaser::MemReleaser()
: X3DMemReleaser()
{
  stateVariables = GraphTraversal::getInstanceOf<MemReleaserStateVariables>();
  
  graphTraversalMap["Mesh"] = new MESH::DFSGraphTraversal();
  graphTraversalMap["Mesh"]->setComponentVisitor(new MESH::MemReleaserCoreVisitor());  
  
  graphTraversalMap["Generic"] = mergeGraphTraversals(new X3D::DFSGraphTraversal(), new GL::DFSGraphTraversal());
  graphTraversalMap["Generic"] = mergeGraphTraversals(graphTraversalMap["Generic"], new MESH::DFSGraphTraversal());
  graphTraversalMap["Generic"]->setComponentVisitor(new MemReleaserGenericAbstractVisitor());
}

MemReleaser::~MemReleaser()
{
  GraphTraversal::removeInstanceOf<MemReleaserStateVariables>();
  for (MGraphTraversal::iterator it = graphTraversalMap.begin(); it != graphTraversalMap.end(); ++it)
    delete (*it).second;
}

void MemReleaser::release(SFAbstractNode N)
{
  graphTraversalMap["Mesh"]->traverse(N);
  
  stateVariables->init();
  graphTraversalMap["Generic"]->traverse(N);
  stateVariables->finish();
}

}
