#include "MemReleaserGenericAbstractVisitor.h"
#include "X3D_Scene.h"
#include "X3DAbstractNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {

MemReleaserGenericAbstractVisitor::MemReleaserGenericAbstractVisitor()
: AbstractVisitor()
{
  // Enter functions.
  defineEnterFunction(&MemReleaserGenericAbstractVisitor::enterX3DAbstractNode);
  
  // Walk on functions.
  defineWalkOnFunction(&MemReleaserGenericAbstractVisitor::walkOnX3DAbstractNode);  
    
  // Leave functions.
  defineLeaveFunction(&MemReleaserGenericAbstractVisitor::leaveX3DAbstractNode);

  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<MemReleaserStateVariables>();
}

MemReleaserGenericAbstractVisitor::~MemReleaserGenericAbstractVisitor()
{
}

void MemReleaserGenericAbstractVisitor::enterX3DAbstractNode(X3DAbstractNode *N) const
{
  // if here, then the parent has to be deleted.
  SFAbstractNode P = stateVariables->getParent();
  
  if (P != 0)
    P->removeChild(N);
    
  if (N->getParentList().size() == 0)
    stateVariables->setDelete(true);
  else
    stateVariables->setDelete(false);
      
  stateVariables->pushParent(N);
}

bool MemReleaserGenericAbstractVisitor::walkOnX3DAbstractNode(X3DAbstractNode *, SFAbstractNode) const
{
  return stateVariables->getDelete();
}

void MemReleaserGenericAbstractVisitor::leaveX3DAbstractNode(X3DAbstractNode *N) const
{
  stateVariables->popParent();
    
  if (stateVariables->getDelete())
    delete N;
  else
    stateVariables->setDelete(true);
}

}
