#include "SimpleX3DGLScene.h"
#include "X3D_Loader.h"
#include "X3D_BBoxUpdater.h"
#include "X3D_GLBuilder.h"
#include "GL_Renderer.h"
#include "MemReleaser.h"
#include "X3D_X3DFileValidator.h"
#include "X3D_Scene.h"
#include "GL_Scene.h"
#include "BBox.h"

#include <iostream>
#include <time.h>

using namespace std;

namespace X3DTK {

SimpleX3DGLScene::SimpleX3DGLScene()
: scene(0), glscene(0)
{
  _loader = X3DLoader::getInstanceOf<X3D::Loader>();
  _bboxupdater = X3DProcessor::getInstanceOf<X3D::BBoxUpdater>();
  _builder = X3DProcessor::getInstanceOf<X3D::GLBuilder>();
  _renderer = X3DProcessor::getInstanceOf<GL::Renderer>();
  _releaser = X3DProcessor::getInstanceOf<MemReleaser>();
}

SimpleX3DGLScene::~SimpleX3DGLScene()
{
  _releaser->release(scene);
  _releaser->release(glscene);  
  scene = 0;
  glscene = 0;
}

void SimpleX3DGLScene::setLoader(X3D::Loader *loader)
{
  _loader = loader;
}

void SimpleX3DGLScene::setBBoxUpdater(X3D::X3DBBoxUpdater *bboxUpdater)
{
  _bboxupdater = bboxUpdater;
}

void SimpleX3DGLScene::setGLBuilder(X3D::X3DGLBuilder *builder)
{
  _builder = builder;
}

void SimpleX3DGLScene::setGLRenderer(GL::X3DRenderer *renderer)
{
  _renderer = renderer;
}

void SimpleX3DGLScene::setMemReleaser(X3DMemReleaser *memReleaser)
{
  _releaser = memReleaser;
}
  
void SimpleX3DGLScene::setFileValidator(X3D::X3DFileValidator *fileValidator)
{
//  _loader->setFileValidator(fileValidator);
}
  
void SimpleX3DGLScene::load(const char *file, bool fileValidation)
{
  loadFile(file, fileValidation);
  computeBBox();
  buildGLScene();
}

void SimpleX3DGLScene::draw()
{
  _renderer->render(glscene);
}

void SimpleX3DGLScene::release()
{
  _releaser->release(scene);
  _releaser->release(glscene);
  scene = 0;
  glscene = 0;  
}

void SimpleX3DGLScene::loadFile(const char *file, bool fileValidation)
{
  _releaser->release(scene);
  _releaser->release(glscene);  
  scene = 0;
  glscene = 0;
  
  scene = _loader->load(file, fileValidation);
}

void SimpleX3DGLScene::computeBBox()
{
  _bboxupdater->update(scene, true);
  
  if (scene != 0)
  {
    SFVec3f center = scene->getBBoxCenter();
    SFVec3f size = scene->getBBoxSize();

    SFVec3f A = center + 0.5f*size;
    SFVec3f B = center - 0.5f*size;

    if (A.x < B.x)
    {
      min.x = A.x;
      max.x = B.x;
    }  
    else
    {
      min.x = B.x;  
      max.x = A.x;
    }  
  
    if (A.y < B.y)
    {
      min.y = A.y;
      max.y = B.y;
    }  
    else
    {
      min.y = B.y;  
      max.y = A.y;
    }
  
    if (A.z < B.z)
    {
      min.z = A.z;
      max.z = B.z;
    }  
    else
    {
      min.z = B.z;  
      max.z = A.z;
    }  
  }  
}

void SimpleX3DGLScene::buildGLScene()
{
  glscene = _builder->build(scene);
}


}
